@@ -4,25 +4,6 @@ package MIME::Parser::Reader;
 
 MIME::Parser::Reader - a line-oriented reader for a MIME::Parser
 
-
-=head1 SYNOPSIS
-
-This module is used internally by MIME::Parser; you probably
-don't need to be looking at it at all.  But just in case...
-
-    ### Create a top-level reader, where chunks end at EOF:
-    $rdr = MIME::Parser::Reader->new();
-      
-    ### Spawn a child reader, where chunks also end at a boundary:
-    $subrdr = $rdr->spawn->add_boundary($bound);
-     
-    ### Spawn a child reader, where chunks also end at a given string:
-    $subrdr = $rdr->spawn->add_terminator($string);
-     
-    ### Read until boundary or terminator:
-    $subrdr->read_chunk($in, $out);
-
-
 =head1 DESCRIPTION
 
 A line-oriented reader which can deal with virtual end-of-stream
@@ -40,15 +21,11 @@ use IO::ScalarArray;
 ### Note that "" is included because last line of stream may have no newline!
 my @EOLs = ("", "\r", "\n", "\r\n", "\n\r");
 
-### Long line:
-my $LONGLINE = ' ' x 1000;
-
 
 #------------------------------
 #
 # new
 #
-# I<Class method.>
 # Construct an empty (top-level) reader.
 #
 sub new {
@@ -57,7 +34,6 @@ sub new {
     return bless {
 	Bounds => [],
 	BH     => {},
-	TH     => {},
 	EOS    => \$eos,
     }, $class;
 }
@@ -66,7 +42,6 @@ sub new {
 #
 # spawn
 #
-# I<Instance method.>
 # Return a reader which is mostly a duplicate, except that the EOS 
 # accumulator is shared.
 #
@@ -75,7 +50,6 @@ sub spawn {
     my $dup = bless {}, ref($self);
     $dup->{Bounds} = [ @{$self->{Bounds}} ];  ### deep copy
     $dup->{BH}     = { %{$self->{BH}} };      ### deep copy
-    $dup->{TH}     = { %{$self->{TH}} };      ### deep copy
     $dup->{EOS}    = $self->{EOS};            ### shallow copy; same ref!
     $dup;
 }
@@ -84,14 +58,15 @@ sub spawn {
 #
 # add_boundary BOUND
 #
-# I<Instance method.>
 # Let BOUND be the new innermost boundary.  Returns self.
 #
 sub add_boundary {
     my ($self, $bound) = @_;
     unshift @{$self->{Bounds}}, $bound;   ### now at index 0
-    $self->{BH}{"--$bound"}   = "DELIM $bound";
-    $self->{BH}{"--$bound--"} = "CLOSE $bound";
+    foreach my $eol (@EOLs) {
+	$self->{BH}{"--$bound$eol"}   = "DELIM $bound";
+	$self->{BH}{"--$bound--$eol"} = "CLOSE $bound";
+    }
     $self;
 }
 
@@ -99,13 +74,12 @@ sub add_boundary {
 #
 # add_terminator LINE
 #
-# I<Instance method.>
 # Let LINE be another terminator.  Returns self.
 #
 sub add_terminator {
     my ($self, $line) = @_;
-    foreach (@EOLs) {
-	$self->{TH}{"$line$_"} = "DONE $line";
+    foreach my $eol (@EOLs) {
+	$self->{BH}{"$line$eol"} = "DONE $line";
     }
     $self;
 }
@@ -114,7 +88,6 @@ sub add_terminator {
 #
 # has_bounds
 #
-# I<Instance method.>
 # Are there boundaries to contend with?
 #
 sub has_bounds {
@@ -125,7 +98,6 @@ sub has_bounds {
 #
 # depth
 #
-# I<Instance method.>
 # How many levels are there? 
 #
 sub depth {
@@ -136,7 +108,6 @@ sub depth {
 #
 # eos [EOS]
 #
-# I<Instance method.>
 # Return the last end-of-stream token seen.
 # See read_chunk() for what these might be.
 #
@@ -150,7 +121,6 @@ sub eos {
 #
 # eos_type [EOSTOKEN]
 #
-# I<Instance method.>
 # Return the high-level type of the given token (defaults to our token).
 #
 #    DELIM       saw an innermost boundary like --xyz
@@ -178,14 +148,13 @@ sub eos_type {
 #
 # native_handle HANDLE
 #
-# I<Function.>
 # Can we do native i/o on HANDLE?  If true, returns the handle
 # that will respond to native I/O calls; else, returns undef.
 #
 sub native_handle {
     my $fh = shift;
-    return $fh if $fh->isa('IO::File');
-    return $fh if (ref $fh eq 'GLOB'); 
+    return $fh  if $fh->isa('IO::File');
+    return $$fh if ($fh->isa('IO::Wrap') && (ref($$fh) eq 'GLOB'));
     undef;
 }
 
@@ -193,7 +162,6 @@ sub native_handle {
 #
 # read_chunk INHANDLE, OUTHANDLE
 #
-# I<Instance method.>
 # Get lines until end-of-stream.
 # Returns the terminating-condition token:
 #
@@ -201,33 +169,16 @@ sub native_handle {
 #    CLOSE xyz   saw boundary line "--xyz--"
 #    DONE xyz    saw terminator line "xyz"
 #    EOF         end of file
-
-# Parse up to (and including) the boundary, and dump output.
-# Follows the RFC 2046 specification, that the CRLF immediately preceding 
-# the boundary is part of the boundary, NOT part of the input!
 #
-# NOTE: while parsing, we take care to remember the EXACT end-of-line
-# sequence.  This is because we *may* be handling 'binary' encoded data, and 
-# in that case we can't just massage \r\n into \n!  Don't worry... if the
-# data is styled as '7bit' or '8bit', the "decoder" will massage the CRLF
-# for us.  For now, we're just trying to chop up the data stream.
-
-# NBK - Oct 12, 1999
-# The CRLF at the end of the current line is considered part
-# of the boundary.  I buffer the current line and output the
-# last.  I strip the last CRLF when I hit the boundary.
-
 sub read_chunk {
     my ($self, $in, $out) = @_;
     
     ### Init:
     my %bh = %{$self->{BH}};
-    my %th = %{$self->{TH}}; my $thx = keys %th;
-    local $_ = $LONGLINE;
-    my $maybe;
     my $last = '';
-    my $eos  = '';
-    
+    my $eos = '';
+    local $_ = ' ' x 1000;
+
     ### Determine types:
     my $n_in  = native_handle($in);
     my $n_out = native_handle($out);
@@ -235,46 +186,26 @@ sub read_chunk {
     ### Handle efficiently by type:
     if ($n_in) {
 	if ($n_out) {            ### native input, native output [fastest]
-	    while (<$n_in>) {
-		if (substr($_, 0, 2) eq '--') {
-		    ($maybe = $_) =~ s/[ \t\r\n]+\Z//;
-		    $bh{$maybe} and do { $eos = $bh{$maybe}; last };
-		}
-		$thx and $th{$_} and do { $eos = $th{$_}; last };
-		print $n_out $last; $last = $_; 
-	    }
+	    while (<$n_in>) { 
+		$bh{$_} and do { $eos = $bh{$_}; last };
+		print $n_out $last; $last = $_;  }
 	}
 	else {                   ### native input, OO output [slower]
 	    while (<$n_in>) { 
-		if (substr($_, 0, 2) eq '--') {
-		    ($maybe = $_) =~ s/[ \t\r\n]+\Z//;
-		    $bh{$maybe} and do { $eos = $bh{$maybe}; last };
-		}
-		$thx and $th{$_} and do { $eos = $th{$_}; last };
-		$out->print($last); $last = $_; 
-	    }
+		$bh{$_} and do { $eos = $bh{$_}; last };
+		$out->print($last); $last = $_; }
 	}
     }
     else {
 	if ($n_out) {            ### OO input, native output [even slower]
 	    while (defined($_ = $in->getline)) { 
-		if (substr($_, 0, 2) eq '--') {
-		    ($maybe = $_) =~ s/[ \t\r\n]+\Z//;
-		    $bh{$maybe} and do { $eos = $bh{$maybe}; last };
-		}
-		$thx and $th{$_} and do { $eos = $th{$_}; last };
-		print $n_out $last; $last = $_;  
-	    }
+		$bh{$_} and do { $eos = $bh{$_}; last };
+		print $n_out $last; $last = $_;  }
 	}
 	else {                   ### OO input, OO output [slowest]
 	    while (defined($_ = $in->getline)) { 
-		if (substr($_, 0, 2) eq '--') {
-		    ($maybe = $_) =~ s/[ \t\r\n]+\Z//;
-		    $bh{$maybe} and do { $eos = $bh{$maybe}; last };
-		}
-		$thx and $th{$_} and do { $eos = $th{$_}; last };
-		$out->print($last); $last = $_; 
-	    }
+		$bh{$_} and do { $eos = $bh{$_}; last };
+		$out->print($last); $last = $_; }
 	}
     }
     
@@ -291,17 +222,17 @@ sub read_chunk {
 #
 # read_lines INHANDLE, \@OUTLINES
 #
-# I<Instance method.>
 # Read lines into the given array.
 # 
 sub read_lines {
     my ($self, $in, $outlines) = @_;
-    # TODO: we are also stuck keeping this one for now
     $self->read_chunk($in, IO::ScalarArray->new($outlines));
     shift @$outlines if ($outlines->[0] eq '');   ### leading empty line
+#   foreach (@$outlines) { s/[\r\n]+\Z/\n/ };
     1;
 }
 
 1;
 __END__
 
+
